#ifndef _IP_NAT_PERCONN_H
#define _IP_NAT_PERCONN_H
/* IP NAT per-connection stuff.  These rules are dynamically created
 * by the setup map, and kept in a hash table. */

/* Play with dynamic later; meanwhile if it's good enough for DaveM's
   TCP code... */
#define IP_NAT_HTABLE_SIZE 64
#include "ip_nat.h"
#include <linux/timer.h>

struct ip_nat_perconn;
struct ip_nat_setup;
struct ip_nat_ip_use;

typedef unsigned int ip_nat_fn(struct ip_nat_perconn *, 
			       const struct iphdr *,
			       enum ip_nat_direction);

/* Never fails, returns NF_ACCEPT.  Suitable for use as ip_nat_fn. 
 * You need lock to use this. */
extern unsigned int ip_nat_delete_perconn(struct ip_nat_perconn *perconn,
					  const struct iphdr *,
					  enum ip_nat_direction);

/* As above, but used for timer. */
extern void ip_nat_delete_perconn_timeout(struct ip_nat_perconn *perconn);

/* Expire in this direction.  If expired in both dirs, delete. */
extern unsigned int ip_nat_delete_maybe(struct ip_nat_perconn *perconn,
					const struct iphdr *iph,
					enum ip_nat_direction dir);

/* Null ip_nat_fn: returns NF_ACCEPT. */
extern unsigned int ip_nat_accept(struct ip_nat_perconn *perconn,
				  const struct iphdr *,
				  enum ip_nat_direction);

/* Returns NULL if can't alloc. */
struct ip_nat_perconn *
ip_nat_alloc_perconn(const struct ip_nat_setup *cause,
		     const struct ip_nat_tuple_proto *tuplepr,
		     enum ip_nat_direction direction,
		     struct ip_nat_address_range *source_range,
		     ip_nat_fn *postmangle, ip_nat_fn *relatedpacket,
		     unsigned long timertime, 
		     void (*timerfn)(struct ip_nat_perconn *perconn),
		     void *extra, size_t extrasize);

struct ip_nat_perconn
{
	/* We sit in four hash lists.  Really. */
	struct ip_nat_perconn *by_tuple[2], *by_ips_proto, *by_source;

	/* These are kept separately for each direction. */
	struct {
		/* Tuple to match. */
		struct ip_nat_tuple_proto connection;
		/* Replacement. */
		struct ip_nat_tuple_manip manip;
		/* Are we expired? */
		atomic_t expired;
		/* Creation or last-used time in jiffies. */
		unsigned long lasthit;
		/* Number of hits. */
		unsigned int hitcount;
	} dir[2];

	/* Whoever created us. */
	const struct ip_nat_setup *cause;

	/* Called when a packet matches: returns NF_*. */
	ip_nat_fn *postmangle;

	/* Called for related packets; return ignored. */
	ip_nat_fn *relatedpacket;

	/* Tuple we got allocated. */
	struct ip_nat_tuple_proto allocated_tuple;

	/* Most setups want a timer, so include it here. */
	struct timer_list timeout;
	void (*timerfn)(struct ip_nat_perconn *perconn);
};
#endif /*_IP_NAT_PERCONN_H*/
