static int null_pkt_to_tuple(const void *datah, size_t datalen, 
			     struct ip_nat_tuple *tuple)
{
	return 1;
}

static void null_tuple_to_pkt(struct iphdr *hdr, 
			      size_t len,
			      const struct ip_nat_tuple_manip *tuple,
			      enum ip_nat_direction direction)
{
	return;
}

static int null_in_range(const struct ip_nat_tuple *tuple, 
			 const struct ip_nat_address_range *range)
{
	return 1;
}

static int null_unique_tuple(struct ip_nat_tuple_proto *tuplepr, 
			     const struct ip_nat_address_range *source_range)
{
	struct ip_nat_perconn *perconn
		= *hash_by_ips_proto(tuplepr->tuple.src.ip, 
				     tuplepr->tuple.dst.ip, tuplepr->proto);

	/* If there's any other with this IP pair/proto, we can't
           allocate. */
	while (perconn) {
		struct ip_nat_tuple_proto *tpr
			= &perconn->dir[IP_NAT_OUTGOING].connection;
		if (tpr->tuple.src.ip == tuplepr->tuple.src.ip
		    && tpr->tuple.dst.ip == tuplepr->tuple.dst.ip
		    && tpr->proto == tuplepr->proto)
			return 0;
	}
	return 1;
}

static void null_invert_tuple(struct ip_nat_tuple *tuple)
{
}

static struct ip_nat_proto null_proto = {
	NULL, 0, "unknown", NULL,
	null_pkt_to_tuple, null_tuple_to_pkt, null_in_range, null_unique_tuple,
	null_invert_tuple
};

static struct ip_nat_proto *
find_proto(unsigned int protocol)
{
	struct ip_nat_proto *ret;

	for (ret = ip_nat_prototable[protocol & (MAX_INET_PROTOS-1)]; 
	     ret && ret->proto != protocol; 
	     ret = ret->next);

	return ret ?: &null_proto;
}

void ip_nat_register_proto(struct ip_nat_proto *proto)
{
	MOD_INC_USE_COUNT;
	modreg_prepend(&ip_nat_prototable[proto->proto & (MAX_INET_PROTOS-1)],
		       proto);
}
	
void ip_nat_unregister_proto(struct ip_nat_proto *proto)
{
	MOD_DEC_USE_COUNT;
	modreg_delete(&ip_nat_prototable[proto->proto & (MAX_INET_PROTOS-1)],
		      proto);
}

