#ifndef _IP_NAT_SETUP_H
#define _IP_NAT_SETUP_H
#include "ip_nat.h"

#define IP_NAT_BASE_CTL  	80	/* base for NAT socket options */
#define IP_NAT_SO_INSERT	IP_NAT_BASE_CTL
#define IP_NAT_SO_DELETE	(IP_NAT_BASE_CTL+1)
#define IP_NAT_SO_MAX		IP_NAT_SO_DELETE

#define IP_NAT_SETUP_MAX_NAMELEN 16

struct ip_nat_setup_user
{
	char typename[IP_NAT_SETUP_MAX_NAMELEN];

	enum ip_nat_direction direction;
	struct ip_nat_tuple_proto tupleproto;
	struct ip_nat_tuple_proto mask;

	/* Set ifname to "" for `don't care about interface' */
	char ifname[IFNAMSIZ];

	/* Private setup stuff for this type. */
	char private[0];
};

#ifdef __KERNEL__
struct iphdr;

/* FIXME: Make this 64 bit for modern machines? --RR */
typedef u_int32_t NICE_TYPE;
struct ip_nat_perconn;

#define FIT_TO_TYPE(x,t) (((x)+(sizeof(t))-1)/(sizeof(t)))

struct ip_nat_setup
{
	struct ip_nat_setup *next;
	struct ip_nat_pktinfo info;
	NICE_TYPE mask[FIT_TO_TYPE(sizeof(struct ip_nat_pktinfo), NICE_TYPE)];

	struct ip_nat_setup_type *type;
	u_int32_t precedence;

	char ifname[IFNAMSIZ];
	enum ip_nat_direction direction;

	/* Private setup stuff for this type. */
	char private[0];
};

struct ip_nat_setup_type
{
	struct ip_nat_setup_type *next;

	/* Name of this mapping. */
	const char name[IP_NAT_SETUP_MAX_NAMELEN];

	/* Must be zero to unregister. */
	unsigned int usage;

	/* If you want your module count modified as rules use the type,
	   set this. */
	struct module *me;

	/* Will it accept this setup? */
	int (*check)(const struct ip_nat_setup *, size_t extrasize);

	/* Function to call if we get a match: if NF_ACCEPT and perconn
	 * filled in, checks perconns again. */
	unsigned int (*setup)(const struct iphdr *iph, 
			      const struct ip_nat_pktinfo *pkt,
			      enum ip_nat_direction direction,
			      const struct ip_nat_setup *cause,
			      struct ip_nat_perconn **perconn);

	/* Print out the private part of the setup. */
	int (*print_setup)(char *buffer, const struct ip_nat_setup *setup);

	/* Print out the private part of the perconn (if any). */
	int (*print_perconn)(char *, const struct ip_nat_perconn *);
};

extern int ip_nat_register_setup_type(struct ip_nat_setup_type *me);
extern int ip_nat_unregister_setup_type(struct ip_nat_setup_type *me);

#endif /*__KERNEL__*/
#endif /*_IP_NAT_SETUP_H*/
