/*
 * This is the 1998 rewrite of IP Firewalling, aiming for kernel 2.3.x.
 *
 * Copyright (C) 1998 Paul F. Russell & Michael J. Neuling 
 *
 * This is an example module which is used for filtering on ethernet
 * MAC addresses
 */

#include <linux/module.h> 
#include <linux/skbuff.h> 
#include <linux/if_ether.h> 
#include <net/icmp.h> 
#include "ip_tables.h" 
#include "ip_tables_standard.h"

struct mac_hdr {
    int verdict;
    unsigned char addr[ETH_ALEN];
};

static int ipt_mac_action(struct sk_buff **skb, 
			  const char *indev,
			  const char *outdev,
			  void *voidinfo, 
			  unsigned int pos) 
{
    const struct mac_hdr *info = voidinfo;
    if ((*skb)->mac.raw >= (*skb)->data 
	&& (*skb)->mac.raw < (*skb)->data + (*skb)->len - ETH_HLEN
	&& memcmp((*skb)->mac.ethernet->h_source, info->addr, ETH_ALEN) == 0)
	    return info->verdict;

    /* else goto next entry */
    return pos + 1;
}

static int ipt_mac_checkentry(const void *voidinfo,
			      unsigned int pos, 
			      unsigned int finalpos) 
{ 
    const struct mac_hdr *info = voidinfo;
    printk("%02x:%02x:%02x:%02x:%02x:%02x\n",
	   info->addr[0],
	   info->addr[1],
	   info->addr[2],
	   info->addr[3],
	   info->addr[4],
	   info->addr[5]
	);
    return 1;
}

static struct ipt_action ipt_mac_reg 
= { NULL, "MAC", ipt_mac_action, ipt_mac_checkentry, &__this_module };

int init_module(void)
{
    if (ipt_register_function(&ipt_mac_reg))
	return -EINVAL;
    
    return 0;
}

void cleanup_module(void)
{
    ipt_unregister_function(&ipt_mac_reg);
}
