#ifndef _IP_TABLES_USERSUPP_H
#define _IP_TABLES_USERSUPP_H
#include <netinet/ip.h>
#include <netinet/ip_icmp.h>
#include <netinet/tcp.h>
#include <netinet/udp.h>
#include <net/if.h>
#include <sys/types.h>
#include <stdio.h>
#include <string.h>
#include <errno.h>

#define printk printf

void panic(const char message[], ...) __attribute__((noreturn));
void *kmalloc(size_t size, int pri);
void kfree(void *ptr);
#define kfree_s(p,s) kfree(p)
#define net_ratelimit() 1
void request_module(const char modulename[]);

typedef int rwlock_t;
#define RW_LOCK_UNLOCKED 0;

#define GFP_KERNEL 0
#define GFP_USER 1
#define GFP_ATOMIC 2

typedef struct
{
	int v;
} atomic_t;

#define atomic_read(a) ((a)->v)
#define atomic_set(a,val) ((a)->v = (val))

int copy_from_user(void *to, void *from, size_t size);
int copy_to_user(void *to, void *from, size_t size);

#define MOD_DEC_USE_COUNT do { } while(0)
#define MOD_INC_USE_COUNT do { } while(0)
#define __MOD_INC_USE_COUNT(m) do { } while(0)
#define __MOD_DEC_USE_COUNT(m) do { } while(0)
#define MODULE_PARM(v, t)

#define UMAX(a,b) ((size_t)(a)>(size_t)(b) ? (size_t)(a) : (size_t)(b))

#define IP_OFFSET	0x1FFF		/* "Fragment Offset" part	*/

struct sk_buff 
{
	u_int32_t	nfmark;
	union
	{
		struct iphdr	*iph;
	} nh;
};		

struct device
{
	char name[IFNAMSIZ];
};

typedef unsigned int nf_hookfn(unsigned int hooknum,
			       struct sk_buff **skb,
			       const struct device *in,
			       const struct device *out);

typedef unsigned int nf_cacheflushfn(const void *packet,
				     const struct device *in,
				     const struct device *out,
				     u_int32_t packetcount,
				     u_int32_t bytecount);

struct nf_hook_ops
{
	/* Set these to NULL/zero */
	struct nf_hook_ops *next;
	unsigned int skbs_in_exile;

	/* User fills in from here down. */
	nf_hookfn *hook;
	nf_cacheflushfn *flush;
	int pf;
	int hooknum;
	/* If you want your module count to go up with usage, fill in this,
	 * otherwise NULL. */
	struct module *me;
};

int nf_register_hook(struct nf_hook_ops *hook);

#endif /*_IP_TABLES_USERSUPP_H*/
